@@ -1,6 +1,12 @@
 #include <Rcpp.h>
 #include <RcppParallel.h>
-using namespace Rcpp;
+
+// see https://www.boost.org/doc/libs/1_77_0/libs/math/doc/html/math_toolkit/stat_tut/weg/error_eg.html
+// and https://www.boost.org/doc/libs/1_77_0/libs/math/doc/html/math_toolkit/pol_tutorial/changing_policy_defaults.html
+
+// return NAN for invalid inputs
+#define BOOST_MATH_DOMAIN_ERROR_POLICY ignore_error
+#include <boost/math/distributions.hpp>       // gamma, normal, poisson, binomial, lognormal distributions
 
 #define fuzz 1e-200
 #define huge 1e10
@@ -10,27 +16,16 @@ int i4 (int i, int j, int k, int l, int ii, int jj, int kk);
 
 //--------------------------------------------------------------------------
 
-double gpois (int count, double lambda, int uselog);
-double gbinom(int count, int size, double p, int uselog);
-double gbinomFP (int count, double size, double p, int uselog);
+double gbinom(int count, int size, double p);
 double pski ( int binomN, int count, double Tski, double g);
 
 //--------------------------------------------------------------------------
 
-// distance between two points given by row k in traps and row m in mask
-double dkm (
-        int k, 
-        int m, 
-        RcppParallel::RMatrix<double> traps, 
-        RcppParallel::RMatrix<double> mask
-);
-//--------------------------------------------------------------------------
-
 void convolvemq (
-        int    mm,                                  // number of points on mask 
-        int    kn,                                  // number of points on kernel 
-        int    j,                                   // session number 1..jj 
-        int    edgecode,                            // adjust for incomplete kernel
+        const int    mm,                            // number of points on mask 
+        const int    kn,                            // number of points on kernel 
+        const int    j,                             // session number 1..jj 
+        const int    edgecode,                      // adjust for incomplete kernel
         const  RcppParallel::RMatrix<int> &mqarray, // input [& 2020-10-31]
         std::vector<double> &kernelp,               // p(move|dx,dy) for points in kernel 
         std::vector<double> &pjm                    // return value 
@@ -38,9 +33,9 @@ void convolvemq (
 //--------------------------------------------------------------------------
 
 void convolvemq1 (
-        int    m,                     // initial location on mask
-        int    j,                     // session number 1..jj 
-        int    edgecode,              // 0 none, no action; 1 wrapped, no action; 2 normalize truncated kernel
+        const int    m,                     // initial location on mask
+        const int    j,                     // session number 1..jj 
+        const int    edgecode,              // 0 none, no action; 1 wrapped, no action; 2 normalize truncated kernel
         const  RcppParallel::RMatrix<int> &mqarray, // input [& 2020-10-31]
         const  std::vector<double> &kernelp, // p(move|dx,dy) for points in kernel 
         std::vector<int>    &mj,
@@ -49,297 +44,301 @@ void convolvemq1 (
 //--------------------------------------------------------------------------
 
 void fillkernelp (
-        int                              jj, 
-        int                              kerneltype, 
-        bool                             sparsekernel,
-        double                           cellsize,
-        double                           r0,
+        const int                        jj, 
+        const int                        kerneltype, 
+        const bool                       sparsekernel,
+        const double                     cellsize,
+        const double                     r0,
         const RcppParallel::RMatrix<int> kernel, 
         const RcppParallel::RVector<int> moveargsi, 
         const std::string                fnname,
         const std::vector<double>        &moveargs, 
         std::vector<double>              &kernelp,
-        bool                             normalize,
-        int                              grain,
+        const bool                       normalize,
+        const int                        grain,
         int                              &returncode
 );
 
 //--------------------------------------------------------------------------
 
 double hfn (
-        int                                 k,                         
-        int                                 m, 
-        int                                 c, 
+        const int                           k,                         
+        const int                           m, 
+        const int                           c, 
         const RcppParallel::RMatrix<double> openval,  
         const RcppParallel::RMatrix<double> traps,
         const RcppParallel::RMatrix<double> mask, 
-        int                                 sigmai, 
-        int                                 detectfn);
+        const int                           sigmai, 
+        const int                           detectfn);
 //--------------------------------------------------------------------------
 
 double hfnd (
-        int k, 
-        int m, 
-        int c, 
+        const int k, 
+        const int m, 
+        const int c, 
         const RcppParallel::RMatrix<double> openval,  
         const RcppParallel::RMatrix<double> distmat,
-        int sigmai, 
-        int detectfn
+        const int sigmai, 
+        const int detectfn
 );
 //--------------------------------------------------------------------------
 
 void getp (
-        int n, 
-        int x, 
-        int nc, 
-        int ss, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int ss, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIA, 
+        const RcppParallel::RVector<int>    PIA, 
         std::vector<double> &p
 );
 //--------------------------------------------------------------------------
 
 void getphij (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
+        const RcppParallel::RVector<int>    PIAJ,
         const RcppParallel::RVector<double> intervals, 
         std::vector<double> &phij
 );
 //--------------------------------------------------------------------------
 
 void getmoveargs (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
-        const RcppParallel::RVector<int> moveargsi,
+        const RcppParallel::RVector<int>    PIAJ,
+        const RcppParallel::RVector<int>    moveargsi,
         std::vector<double> &moveargs
 );
 //--------------------------------------------------------------------------
 
 void getpj (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
+        const RcppParallel::RVector<int>    PIAJ,
         std::vector<double> &pj
 );
 //--------------------------------------------------------------------------
 
 void getg (
-        int type, 
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int type, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
+        const RcppParallel::RVector<int>    PIAJ,
         std::vector<double> &g
 );
 //--------------------------------------------------------------------------
 
 void getfj (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
+        const RcppParallel::RVector<int>    PIAJ,
         const RcppParallel::RVector<double> intervals, 
-        std::vector<double> &phij,
+        const std::vector<double> phij,
         std::vector<double> &fj
 );
 //--------------------------------------------------------------------------
 
 void getlj (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
+        const RcppParallel::RVector<int>    PIAJ,
         const RcppParallel::RVector<double> intervals, 
         std::vector<double> &lj
 );
 //--------------------------------------------------------------------------
 
 void getgaml (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
+        const RcppParallel::RVector<int>    PIAJ,
         const RcppParallel::RVector<double> intervals, 
         std::vector<double> &gam);
 //--------------------------------------------------------------------------
 
 void getgamj (
-        int n, int x, int nc, int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
+        const RcppParallel::RVector<int>    PIAJ,
         const RcppParallel::RVector<double> intervals, 
         std::vector<double> &gamj
 );
 //--------------------------------------------------------------------------
 
 void getkapj (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
+        const RcppParallel::RVector<int>    PIAJ,
         std::vector<double> &kapj
 );
 //--------------------------------------------------------------------------
 
 void getbeta0 (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
+        const RcppParallel::RVector<int>    PIAJ,
         std::vector<double> &beta
 );
 //--------------------------------------------------------------------------
 
 void gettau (
-        int n, 
-        int x, 
-        int nc, 
-        int jj,
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj,
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
+        const RcppParallel::RVector<int>    PIAJ,
         std::vector<double> &tau,
-        int M
+        const int M
 );
 //--------------------------------------------------------------------------
 
 void getDj (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
+        const RcppParallel::RVector<int>    PIAJ,
         std::vector<double> &Dj
 );
 //--------------------------------------------------------------------------
 
 // per capita recruitment cf Link & Barker 2005, Schwarz 'Gentle Intro'
 void getbetaf (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
-        std::vector<double> &phij,
+        const RcppParallel::RVector<int>    PIAJ,
+        const std::vector<double> phij,
         const RcppParallel::RVector<double> intervals, 
         std::vector<double> &beta
 );
 //--------------------------------------------------------------------------
 
 void getbetal (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ,
-        std::vector<double> &phij, 
+        const RcppParallel::RVector<int>    PIAJ,
+        const std::vector<double> phij, 
         const RcppParallel::RVector<double> intervals, 
         std::vector<double> &beta
 );
 //--------------------------------------------------------------------------
 
 void getbetag (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ, 
-        std::vector<double> &phij, 
+        const RcppParallel::RVector<int>    PIAJ, 
+        const std::vector<double> phij, 
         const RcppParallel::RVector<double> intervals, 
         std::vector<double> &beta
 );
 //--------------------------------------------------------------------------
 
 void getbetak (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ, 
-        std::vector<double> &phij,
+        const RcppParallel::RVector<int>    PIAJ, 
+        const std::vector<double> phij,
         std::vector<double> &beta
 );
 //--------------------------------------------------------------------------
 
 // return parameterisation cf Pledger et al. 2010 p 885 
 void getbetaB (
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ, 
+        const RcppParallel::RVector<int>    PIAJ, 
         std::vector<double> &beta
 );
 //--------------------------------------------------------------------------
 
 void getbetaD (
-        int n, 
-        int x,
-        int nc, 
-        int jj, 
+        const int n, 
+        const int x,
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ, 
-        std::vector<double> &phij,
+        const RcppParallel::RVector<int>    PIAJ, 
+        const std::vector<double> phij,
         std::vector<double> &beta
 );
 //--------------------------------------------------------------------------
 
 void getbeta (
-        int type, 
-        int n, 
-        int x, 
-        int nc, 
-        int jj, 
+        const int type, 
+        const int n, 
+        const int x, 
+        const int nc, 
+        const int jj, 
         const RcppParallel::RMatrix<double> openval,  
-        const RcppParallel::RVector<int> PIAJ, 
+        const RcppParallel::RVector<int>    PIAJ, 
         const RcppParallel::RVector<double> intervals,
-        std::vector<double> &phij,
+        const std::vector<double> phij,
         std::vector<double> &beta
 );
 //--------------------------------------------------------------------------
 
-List makelookupcpp (const NumericMatrix x);  
+Rcpp::List makelookupcpp (const Rcpp::NumericMatrix x); 
+
 //--------------------------------------------------------------------------
 
 void pr0njmx (
-        int n, 
-        int x, 
+        const int n, 
+        const int x, 
         const RcppParallel::RVector<int> cumss, 
-        int nc,  
-        int jj, 
-        int kk, 
-        int mm, 
-        int cc0, 
-        int binomN,  
-        const RcppParallel::RVector<int> PIA0, 
+        const int nc,  
+        const int jj, 
+        const int kk, 
+        const int mm, 
+        const int cc0, 
+        const int binomN,  
+        const RcppParallel::RVector<int>    PIA0, 
         const RcppParallel::RVector<double> gk0, 
         const RcppParallel::RMatrix<double> Tsk, 
         std::vector<double> &pjm
